package com.cmeeting.controller;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.cmeeting.ad.entity.RobotSecurityUser;
import com.cmeeting.ad.util.SecurityUtil;
import com.cmeeting.email.EmailSender;
import com.cmeeting.mapper.primary.MeetingRecordTemplateMapper;
import com.cmeeting.mapper.primary.UserIdMapper;
import com.cmeeting.pojo.MeetingInfo;
import com.cmeeting.pojo.MeetingRecordTemplate;
import com.cmeeting.service.MeetingInfoService;
import com.cmeeting.util.MinioUtils;
import com.cmeeting.util.R;
import com.cmeeting.vo.EmailPush;
import com.cmeeting.vo.MeetingInfoVO;
import com.deepoove.poi.XWPFTemplate;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.dataformat.xml.XmlMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.apache.poi.xwpf.usermodel.XWPFParagraph;
import org.apache.poi.xwpf.usermodel.XWPFRun;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.ClassPathResource;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

@Slf4j
@RestController
@RequestMapping("/meetingInfo")
public class MeetingInfoController {

    @Resource
    private MeetingInfoService meetingInfoService;
    @Resource
    private MinioUtils minioUtils;
    @Resource
    private EmailSender emailSender;
    @Resource
    private MeetingRecordTemplateMapper meetingRecordTemplateMapper;
    @Resource
    private UserIdMapper userIdMapper;
    @Value(value = "${permission.applicationId}")
    private String permissionApplicationId;
    @Value(value = "${userAdmin.file-download-path}")
    private String fileDownloadPath;

    @PostMapping("/updateRecordXml")
    public R updateRecordXml(@RequestBody MeetingInfoVO vo) {
        boolean save = meetingInfoService.updateRecordXml(vo);
        return R.ok(save);
    }

    /**
     * 重新生成
     * @param vo
     * @return
     */
    @PostMapping("/regenerateXml")
    public R regenerateXml(@RequestBody MeetingInfoVO vo) {
        boolean isSuccess = meetingInfoService.regenerateXml(vo);
        return R.ok(isSuccess);
    }

    @PostMapping("/list")
    public R list(@RequestBody MeetingInfoVO vo) {
        IPage<MeetingInfo> page = meetingInfoService.getPage(vo);
        return R.ok(page);
    }

    @PostMapping("/detail")
    public R detail(@RequestBody MeetingInfoVO vo) {
        RobotSecurityUser user = SecurityUtil.getUser();
        vo.setUserId(user.getId());
        vo.setApplicationId(permissionApplicationId);
        MeetingInfo meetingInfo = meetingInfoService.getById(vo.getId());
        BeanUtils.copyProperties(meetingInfo,vo);
        String recordXml = vo.getRecordXml();
        String recordContent = vo.getRecordContent();
        try {
            if(StringUtils.isNotEmpty(recordXml)){
                //xml转json，用于前端的表单回显
                String xml = minioUtils.getFileText(recordXml);
                String json = convertXmlToJSON(xml);
                vo.setRecordJson(json);
            }
        } catch (Exception e) {
            e.printStackTrace();
//            try(InputStream is = minioUtils.getFile(recordXml)){
//                byte[] bytes = IOUtils.toByteArray(is);
//                String json = convertXmlToJSON(new String(bytes));
//                vo.setRecordJson(json);
//            }catch (Exception ex){
//                e.printStackTrace();
//            }
        }
        try {
            if(StringUtils.isNotEmpty(recordContent)){
                vo.setRecordContent(minioUtils.getFileText(recordContent.replaceAll(fileDownloadPath,"")));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return R.ok(vo);
    }

    /**
     * 导出转写原文
     */
    @PostMapping("/exportMeetingRecord")
    public void exportMeetingRecord(@RequestBody MeetingInfoVO vo, HttpServletResponse response){
        try {
            MeetingInfo meetingInfo = meetingInfoService.getById(vo.getId());
            String content = minioUtils.getFileText(meetingInfo.getRecordContent());

            String fileName = String.format(meetingInfo.getSubject() + "_转写原文_%s.docx", DateUtil.today());
            response.setContentType("application/vnd.openxmlformats-officedocument.wordprocessingml.document");
            response.setCharacterEncoding("utf-8");
            response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "utf8"));
            try (XWPFDocument document = new XWPFDocument(); OutputStream os = response.getOutputStream()) {
                // 将文本按行分割并逐行添加到 Word 文档
                String[] lines = content.split("\\r?\\n");
                for (String line : lines) {
                    XWPFParagraph paragraph = document.createParagraph();
                    XWPFRun run = paragraph.createRun();
                    run.setText(line);
                }

                // 输出文档
                document.write(os);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        } catch (Exception e) {
            response.reset();
            response.setContentType("text/plain");
            response.setCharacterEncoding("utf-8");
            e.printStackTrace();
        }
    }

    /**
     * 导出会议纪要（本地下载）
     */
    @PostMapping("/downloadMeetingMinutes")
    public void downloadMeetingMinutes(@RequestBody MeetingInfoVO vo, HttpServletResponse response){
        try {
            MeetingInfo meetingInfo = meetingInfoService.getById(vo.getId());
            String xml = minioUtils.getFileText(meetingInfo.getRecordXml());
            Map<String, Object> dataModel = convertXmlToMap(xml);
            //追加参会人员信息
            Map<String,Object> participantsMap = new ConcurrentHashMap<>();
            DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            String meetingDate = meetingInfo.getStartTime().toLocalDate().format(df);
            participantsMap.put("meeting_date",meetingDate);
            participantsMap.put("meeting_location","线上腾讯会议");
            participantsMap.put("meeting_participants", meetingInfo.getParticipantUsers());
            participantsMap.put("meeting_host",meetingInfo.getHost());

            dataModel.putAll(participantsMap);
            MeetingRecordTemplate meetingRecordTemplate = meetingRecordTemplateMapper.selectById(2);
            String fileName = String.format(meetingInfo.getSubject() + "_会议纪要_%s.docx", DateUtil.today());
            response.setContentType("application/vnd.openxmlformats-officedocument.wordprocessingml.document");
            response.setCharacterEncoding("utf-8");
            response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(fileName, "utf8"));
            XWPFTemplate template;
            try (InputStream is = minioUtils.getFile(meetingRecordTemplate.getTemplate());OutputStream os = response.getOutputStream()) {
                template = XWPFTemplate.compile(is).render(dataModel);
                template.write(os);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        } catch (Exception e) {
            response.reset();
            response.setContentType("text/plain");
            response.setCharacterEncoding("utf-8");
            e.printStackTrace();
        }
    }

    /**
     * 导出会议纪要（推送邮件）
     */
    @PostMapping("/pushMeetingMinutes")
    public R exportMeetingMinutes(@RequestBody MeetingInfoVO vo, HttpServletResponse response){
        try {
            MeetingInfo meetingInfo = meetingInfoService.getById(vo.getId());
            String xml = minioUtils.getFileText(meetingInfo.getRecordXml());
            Map<String, Object> dataModel = convertXmlToMap(xml);
            //追加参会人员信息
            Map<String,Object> participantsMap = new ConcurrentHashMap<>();
            DateTimeFormatter df = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            String meetingDate = meetingInfo.getStartTime().toLocalDate().format(df);
            participantsMap.put("meeting_date",meetingDate);
            participantsMap.put("meeting_location","线上腾讯会议");
            participantsMap.put("meeting_participants", meetingInfo.getParticipantUsers());
            participantsMap.put("meeting_host",meetingInfo.getHost());

            dataModel.putAll(participantsMap);
            MeetingRecordTemplate meetingRecordTemplate = meetingRecordTemplateMapper.selectById(2);
            XWPFTemplate template;
            byte[] meetingMinutesBytes;
            try (InputStream is = minioUtils.getFile(meetingRecordTemplate.getTemplate());ByteArrayOutputStream baos = new ByteArrayOutputStream()) {
                template = XWPFTemplate.compile(is).render(dataModel);
                template.write(baos);
                meetingMinutesBytes = baos.toByteArray();
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
            List<EmailPush.Attachment> attachments = new ArrayList<>();
            EmailPush.Attachment attachment = EmailPush.Attachment.builder().name(meetingInfo.getSubject()+"_数据网络中心").bytes(meetingMinutesBytes).build();
            attachments.add(attachment);
            String hostUid = meetingInfo.getHostUid();
            String userCode = userIdMapper.getWidByTid(hostUid);
            EmailPush emailPushBuilder = EmailPush.builder()
                    .toEmail(meetingInfo.getEmail())
                    .meetingId(meetingInfo.getMeetingId())
                    .attachments(attachments)
                    .subject(meetingInfo.getSubject())
                    .meetingInstanceId(meetingInfo.getId())
                    .subMeetingId(meetingInfo.getSubMeetingId())
                    .toUserCode(userCode)
                    .toUser(meetingInfo.getHost())
                    .build();
            emailSender.sendEmailWithAttachment(emailPushBuilder);
            return R.ok("发送纪要邮件成功");
        } catch (Exception e) {
            response.reset();
            response.setContentType("text/plain");
            response.setCharacterEncoding("utf-8");
            e.printStackTrace();
            return R.error("发送纪要邮件失败");
        }
    }

    public static Map<String, Object> convertXmlToMap(String xml) throws Exception {

        XmlMapper xmlMapper = new XmlMapper();
        ObjectMapper objectMapper = new ObjectMapper();

        // 先将 XML 读取为 Map
        Map<?, ?> xmlMap = xmlMapper.readValue(xml, Map.class);

        // 转换为更标准的 Map<String, Object>
        Map<String,Object> map = objectMapper.convertValue(xmlMap, Map.class);
        //特殊处理map格式
        for (Map.Entry<String, Object> entry : map.entrySet()) {
            Map<String,Object> value = (Map<String, Object>) entry.getValue();
            Object realValue = value.get("");
            entry.setValue(realValue);
        }
        return map;
    }

    private String convertXmlToJSON(String xml) {
        String json;
        try {
            XmlMapper xmlMapper = new XmlMapper();
            JsonNode rootNode = xmlMapper.readTree(xml.getBytes());

            // 正确获取节点和属性的方式
            List<Map> list = new ArrayList<>();
            Iterator<String> iterator = rootNode.fieldNames();
            while (iterator.hasNext()){
                String tagName = iterator.next();
                JsonNode subNode = rootNode.path(tagName);
                String displayName = subNode.path("label").asText();
                String content = subNode.path("").asText();
                list.add(new HashMap(){{
                    put("key",tagName);
                    put("keyName",displayName);
                    put("value",content);
                }});
            }
            // 构建JSON对象
            ObjectMapper jsonMapper = new ObjectMapper();
            json = jsonMapper.writeValueAsString(list);
        } catch (IOException e) {
            log.error(e.getMessage());
            throw new RuntimeException(e.getMessage());
        }
        return json;
    }
}
