package com.cmeeting.controller;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.cmeeting.ad.entity.RobotSecurityUser;
import com.cmeeting.ad.util.SecurityUtil;
import com.cmeeting.dto.UserDTO;
import com.cmeeting.pojo.MeetingRecordTemplate;
import com.cmeeting.service.MeetingRecordTemplateService;
import com.cmeeting.service.RecordTemplatePermissionService;
import com.cmeeting.util.R;
import com.cmeeting.vo.RecordTemplateVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.List;


@RestController
@RequestMapping("/recordTemplate")
public class RecordTemplateController {
    @Autowired
    private MeetingRecordTemplateService recordTemplateService;
    @Resource
    private RecordTemplatePermissionService recordTemplatePermissionService;

    @PostMapping("/create")
    public R create(@RequestBody MeetingRecordTemplate recordTemplate) {
        RobotSecurityUser user = SecurityUtil.getUser();
        LocalDateTime now = LocalDateTime.now();
        recordTemplate.setCreateTime(now);
        recordTemplate.setUpdateTime(now);
        recordTemplate.setIsDel(Boolean.FALSE);
        recordTemplate.setCreateUser(user.getId());
        recordTemplate.setUpdateUser(user.getId());
        recordTemplate.setCreateUserName(user.getNickName());
        recordTemplate.setUpdateUserName(user.getNickName());
        boolean save = recordTemplateService.save(recordTemplate);
        return R.ok(save);
    }

    @PostMapping("/update")
    public R update(@RequestBody MeetingRecordTemplate recordTemplate) {
        RobotSecurityUser user = SecurityUtil.getUser();
        recordTemplate.setUpdateTime(LocalDateTime.now());
        recordTemplate.setUpdateUser(user.getId());
        recordTemplate.setUpdateUserName(user.getNickName());
        boolean save = recordTemplateService.updateById(recordTemplate);
        return R.ok(save);
    }

    @PostMapping("/disable")
    public R disable(@RequestBody MeetingRecordTemplate recordTemplate) {
        boolean save = recordTemplateService.update(
                new LambdaUpdateWrapper<MeetingRecordTemplate>()
                        .eq(MeetingRecordTemplate::getId,recordTemplate.getId())
                        .set(MeetingRecordTemplate::getIsDel,Boolean.TRUE)
        );
        return R.ok(save);
    }

    /**
     * 纪要模板列表，通过type区分为系统模板还是自定义模板
     * @param vo
     * @return
     */
    @PostMapping("/list")
    public R list(@RequestBody RecordTemplateVO vo) {
        IPage<MeetingRecordTemplate> page = recordTemplateService.getPage(vo);
        return R.ok(page);
    }

    /**
     * 用户下拉选择的模板列表（权限内）
     * @param vo
     * @return
     */
    @PostMapping("/selectList")
    public R selectList(@RequestBody RecordTemplateVO vo) {
        List<MeetingRecordTemplate> records = recordTemplateService.getSelectList(vo);
        return R.ok(records);
    }

    /**
     * 普通用户纪要模板，查出有授权的系统模板和个人创建的自定义模板
     * @param vo
     * @return
     */
    @PostMapping("/personalList")
    public R personalList(@RequestBody RecordTemplateVO vo) {
        IPage<MeetingRecordTemplate> page = recordTemplateService.getPersonalPage(vo);
        return R.ok(page);
    }

    @PostMapping("/detail")
    public R detail(@RequestBody RecordTemplateVO vo) {
        MeetingRecordTemplate detail = recordTemplateService.getById(vo.getId());
        return R.ok(detail);
    }

    @PostMapping("/templateUpload")
    public R templateUpload(@RequestParam("file") MultipartFile file,@RequestParam("id") Integer id) {
        String template = recordTemplateService.templateUpload(file, id);
        return R.ok(template);
    }

    /**
     * 模板授权
     * @param vo
     * @return
     */
    @PostMapping("/auth")
    public R auth(@RequestBody RecordTemplateVO.TemplatePermissionVO vo) {
        return R.ok(recordTemplatePermissionService.auth(vo));
    }

    /**
     * 模板已授权列表
     * @param vo
     * @return
     */
    @PostMapping("/authList")
    public R authList(@RequestBody RecordTemplateVO.TemplateAuthVO vo) {
        return R.ok(recordTemplatePermissionService.authList(vo));
    }

    /**
     * 获取模板授权的人员
     * @return
     */
    @PostMapping("/selectAuthorizedUsers")
    public R selectAuthorizedUsers() {
        return R.ok(recordTemplateService.selectAuthorizedUsers());
    }
}
