package com.cmeeting.service.impl;

import cn.chatbot.openai.completion.chat.ChatCompletionRequest;
import cn.chatbot.openai.completion.chat.ChatMessage;
import cn.chatbot.openai.completion.chat.ChatMessageRole;
import cn.chatbot.openai.completion.chat.Message;
import cn.chatbot.openai.service.LLMService;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cmeeting.ad.entity.RobotSecurityUser;
import com.cmeeting.ad.util.SecurityUtil;
import com.cmeeting.log.service.ProcessLogService;
import com.cmeeting.mapper.primary.MeetingInfoMapper;
import com.cmeeting.mapper.primary.MeetingRecordTemplateMapper;
import com.cmeeting.pojo.MeetingInfo;
import com.cmeeting.pojo.MeetingRecordTemplate;
import com.cmeeting.service.MeetingInfoService;
import com.cmeeting.util.MinioUtils;
import com.cmeeting.vo.EmailPush;
import com.cmeeting.vo.MeetingInfoVO;
import com.deepoove.poi.XWPFTemplate;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.dataformat.xml.XmlMapper;
import com.tencentcloudapi.wemeet.Client;
import com.tencentcloudapi.wemeet.core.authenticator.AuthenticatorBuilder;
import com.tencentcloudapi.wemeet.core.authenticator.JWTAuthenticator;
import com.tencentcloudapi.wemeet.service.meetings.api.MeetingsApi;
import com.tencentcloudapi.wemeet.service.meetings.model.V1MeetingsMeetingIdParticipantsGet200Response;
import com.tencentcloudapi.wemeet.service.meetings.model.V1MeetingsMeetingIdParticipantsGet200ResponseParticipantsInner;
import com.tencentcloudapi.wemeet.service.records.api.RecordsApi;
import com.tencentcloudapi.wemeet.service.records.model.V1AddressesRecordFileIdGet200Response;
import com.tencentcloudapi.wemeet.service.records.model.V1AddressesRecordFileIdGet200ResponseAiMeetingTranscriptsInner;
import lombok.extern.slf4j.Slf4j;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.*;
import java.math.BigInteger;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.security.SecureRandom;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

@Service
@Slf4j
public class MeetingInfoServiceImpl extends ServiceImpl<MeetingInfoMapper, MeetingInfo>  implements MeetingInfoService {

    @Resource
    private MeetingInfoMapper mapper;
    @Resource
    private MinioUtils minioUtils;
    @Value(value = "${tencent.appId}")
    private String tencentAppId;
    @Value(value = "${tencent.sdkId}")
    private String tencentSdkId;
    @Value(value = "${tencent.secretId}")
    private String tencentSecretId;
    @Value(value = "${tencent.secretKey}")
    private String tencentSecretKey;
    @Value(value = "${tencent.admin.userId}")
    private String tencentAdminUserId;
    @Value(value = "${llm.api-addr}")
    private String llmApiAddr;
    @Value(value = "${tencent.base-save-path}")
    private String baseSavePath;
    @Resource
    private ProcessLogService processLogService;
    @Resource
    private MeetingRecordTemplateMapper meetingRecordTemplateMapper;
    @Resource
    private MeetingInfoMapper meetingInfoMapper;

    @Override
    public IPage<MeetingInfo> getPage(MeetingInfoVO vo) {
        RobotSecurityUser user = SecurityUtil.getUser();
        vo.setUserId(user.getId());
        LambdaQueryWrapper<MeetingInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.like(StringUtils.isNotEmpty(vo.getSubject()),MeetingInfo::getSubject,vo.getSubject());
        queryWrapper.between(vo.getStartTime() != null,MeetingInfo::getStartTime,vo.getStartTime(),vo.getEndTime());
        IPage<MeetingInfo> resultPage = mapper.selectPage(new Page<>(vo.getCurrent(), vo.getSize()), queryWrapper);
        return resultPage;
    }

    @Override
    public boolean updateRecordXml(MeetingInfoVO vo) {
        //前端的表单json转xml，xml格式的纪要内容便于生成会议纪要文件
        String recordJson = vo.getRecordJson();
        String recordXml = convertJSONToXml(recordJson);
        String key = vo.getMeetingId() + "-recordXmlPath-" + IdUtil.fastSimpleUUID() + ".xml";
        minioUtils.upload(key,recordXml.getBytes(StandardCharsets.UTF_8));
        vo.setRecordXml(key);

        LambdaUpdateWrapper<MeetingInfo> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.eq(MeetingInfo::getId,vo.getId());
        updateWrapper.set(MeetingInfo::getUpdateTime,LocalDateTime.now());
        updateWrapper.set(MeetingInfo::getRecordXml,key);
        return update(null,updateWrapper);
    }

    @Override
    public boolean regenerateXml(MeetingInfoVO vo) {
        Client client = new Client.Builder()
                .withAppId(tencentAppId).withSdkId(tencentSdkId)
                .withSecret(tencentSecretId,tencentSecretKey)
                .build();
        MeetingInfo meetingInfo = mapper.selectById(vo.getId());
        //已保存的会议信息
        String meetingId = meetingInfo.getMeetingId();
        String subMeetingId = meetingInfo.getSubMeetingId();
        String recordFileIdArray = meetingInfo.getRecordFileId();
        try{
            if(StringUtils.isEmpty(recordFileIdArray)){
                return false;
            }
            String meetingDate = meetingInfo.getStartTime().toLocalDate().format(DateTimeFormatter.ISO_LOCAL_DATE);

            // 获取参会成员明细
            MeetingsApi.ApiV1MeetingsMeetingIdParticipantsGetRequest participantsRequest =
                    new MeetingsApi.ApiV1MeetingsMeetingIdParticipantsGetRequest
                            .Builder(meetingId).subMeetingId(subMeetingId).operatorId(tencentAdminUserId).operatorIdType("1").build();
            AuthenticatorBuilder<JWTAuthenticator> participantsAuthenticatorBuilder =
                    new JWTAuthenticator.Builder()
                            .nonce(BigInteger.valueOf(Math.abs((new SecureRandom()).nextInt())))
                            .timestamp(String.valueOf(System.currentTimeMillis() / 1000L));
            MeetingsApi.ApiV1MeetingsMeetingIdParticipantsGetResponse participantsResponse =
                    client.meetings().v1MeetingsMeetingIdParticipantsGet(participantsRequest, participantsAuthenticatorBuilder);
            V1MeetingsMeetingIdParticipantsGet200Response participantsData = participantsResponse.getData();
            List<V1MeetingsMeetingIdParticipantsGet200ResponseParticipantsInner> participants = participantsData.getParticipants();
            String participantNames = participants.stream().map(item -> new String(Base64.getDecoder().decode(item.getUserName()))).distinct().collect(Collectors.joining("、"));
            meetingInfo.setParticipantUsers(participantNames);

            //每场会议可能会分段录制，查出每个文件的转录记录后拼接
            StringBuffer recordTextBuffer = new StringBuffer();
            List<String> recordFileIdList = Arrays.asList(meetingInfo.getRecordFileId().split(","));
            for (String recordFileId : recordFileIdList) {
                //查询录制转写详情
                RecordsApi.ApiV1AddressesRecordFileIdGetRequest addressRequest =
                        new RecordsApi.ApiV1AddressesRecordFileIdGetRequest.Builder(recordFileId)
                                .operatorId(tencentAdminUserId)
                                .operatorIdType("1")
                                .build();
                RecordsApi.ApiV1AddressesRecordFileIdGetResponse addressResponse =
                        client.records().v1AddressesRecordFileIdGet(addressRequest,
                                new JWTAuthenticator.Builder().nonce(BigInteger.valueOf(Math.abs((new SecureRandom()).nextInt())))
                                        .timestamp(String.valueOf(System.currentTimeMillis() / 1000L)));
                // 处理响应
                if (addressResponse != null && addressResponse.getData() != null) {
                    log.info("Successfully got address for record file {}", recordFileId);
                    V1AddressesRecordFileIdGet200Response addressData = addressResponse.getData();

                    // 获取AI会议转录文件
                    List<V1AddressesRecordFileIdGet200ResponseAiMeetingTranscriptsInner> transcripts =
                            addressData.getAiMeetingTranscripts();

                    if (transcripts != null && !transcripts.isEmpty()) {
                        log.info("Found {} AI meeting transcripts for record file {}",
                                transcripts.size(), recordFileId);

                        // 处理每个转录文件
                        for (V1AddressesRecordFileIdGet200ResponseAiMeetingTranscriptsInner transcript : transcripts) {
                            String fileType = transcript.getFileType();
                            String downloadUrl = transcript.getDownloadAddress();

                            if ("txt".equalsIgnoreCase(fileType)) {
                                log.info("AI Transcript - Type: {}, URL: {}", fileType, downloadUrl);

                                // 1. 下载文件
                                byte[] fileData = downloadFile(downloadUrl);

                                // 2. 将二进制文件转换为文本
                                String recordTextContent = new String(fileData);
                                if(StringUtils.isNotEmpty(recordTextContent.replaceAll("\\n","").trim())){
                                    recordTextBuffer.append("\n\n");
                                    recordTextBuffer.append(recordTextContent);
                                }
                            }
                        }
                    } else {
                        log.info("No AI meeting transcripts found for record file {}", recordFileId);
                    }
                } else {
                    log.warn("Empty response for record file: {}", recordFileId);
                }
            }
            if(StringUtils.isEmpty(recordTextBuffer.toString().replaceAll("\\n","").trim())){
                log.info("获取的转录文本为空，跳过纪要生成，meetingId:{}，fileRecordId:{}",meetingId,recordFileIdList.toString());
                processLogService.log(meetingId,subMeetingId,"获取的转录文本为空，跳过纪要生成");
                throw new RuntimeException("获取的转录文本为空，跳过纪要生成");
            }
            //获取系统模板
            MeetingRecordTemplate meetingRecordTemplate = meetingRecordTemplateMapper.selectById(2);
            String processedResult = processWithClaude(recordTextBuffer.toString(),meetingDate,meetingRecordTemplate.getPrompt(),meetingId,subMeetingId);
            saveResult(processedResult, recordTextBuffer.toString().getBytes(StandardCharsets.UTF_8), meetingInfo,  meetingRecordTemplate);
            meetingInfoMapper.update(null,
                    new LambdaUpdateWrapper<MeetingInfo>()
                            .eq(MeetingInfo::getMeetingId,meetingId)
                            .eq(subMeetingId != null,MeetingInfo::getSubMeetingId,subMeetingId)
                            .set(MeetingInfo::getRecordXml,meetingInfo.getRecordXml()));
            return true;
        }catch (Exception e){
            StringWriter sw = new StringWriter();
            PrintWriter pw = new PrintWriter(sw);
            e.printStackTrace(pw);
            processLogService.log(meetingId,subMeetingId,sw.toString());
            return false;
        }
    }

    /**
     * 保存会议纪要相关的文件
     * @param content 大模型返回的不规则xml
     * @param recordData 转录文本
     * @param meetingInfo 会议对象
     * @param meetingRecordTemplate 模板信息
     */
    private void saveResult(String content, byte[] recordData, MeetingInfo meetingInfo,MeetingRecordTemplate meetingRecordTemplate) {
        String meetingId = meetingInfo.getMeetingId();
        String subMeetingId = meetingInfo.getSubMeetingId();
        //转录文件临时存储路径
        String recordContentPath = meetingId + "-recordContent-" + IdUtil.fastSimpleUUID() + ".txt";
        //生成的xml临时存储路径
        String recordXmlPath = meetingId + "-recordXmlPath-" + IdUtil.fastSimpleUUID() + ".xml";
        try {
            //去除内容中除了xml内容以外其他的信息，格式化xml
            String xml = extractXmlFromMarkdown(content, meetingId, subMeetingId);
            minioUtils.upload(recordXmlPath,xml.getBytes(StandardCharsets.UTF_8));
            meetingInfo.setRecordXml(recordXmlPath);
        } catch (Exception e) {
            log.error("填充会议纪要失败: {}", e.getMessage(), e);
            StringWriter sw = new StringWriter();
            PrintWriter pw = new PrintWriter(sw);
            e.printStackTrace(pw);
            processLogService.log(meetingId,subMeetingId,"填充会议纪要失败"+sw.toString());
            throw new RuntimeException("填充会议纪要失败");
        }
    }

    /**
     * 大模型生成纪要xml
     * @param textContent 转录文件
     * @param prompt 提示词
     * @return
     */
    private String processWithClaude(String textContent, String meetingDate, String prompt,String meetingId, String subMeetingId) {
        //将文件传送给大模型处理
        String token = "AKIAXFAXF62IWJXGLVEE.LnKInaahcMZG9zLsGMH3nTLOw3S3lK5Vcu0+ifnO";
        String apiAddr = llmApiAddr + "/llm/sse-invoke";
        String model = "anthropic.claude-3-5-sonnet-20240620-v1:0";
        int maxTokens = 5000;
        List<Message> messages = new ArrayList<>();

        ChatMessage chatMessage = new ChatMessage(ChatMessageRole.USER.value(), prompt.replaceAll("\\{transcript\\}",textContent).replaceAll("\\{meetingDate\\}",meetingDate));
        messages.add(chatMessage);
//        chatMessage = new ChatMessage(ChatMessageRole.ASSISTANT.value(), "好的请提供会议记录");
//        messages.add(chatMessage);
//        chatMessage = new ChatMessage(ChatMessageRole.USER.value(), textContent);

        messages.add(chatMessage);

        // 调用Claude API处理文件
        String ret = call_llm(apiAddr, model, token, messages, maxTokens,meetingId,subMeetingId);
        return ret;
    }

    private String call_llm(String apiAddr, String model, String token, List<Message> messages, int maxTokens,String meetingId, String subMeetingId) {
        LLMService service = new LLMService(token, apiAddr);
        StringBuilder stringBuilder = new StringBuilder();
        try {
            ChatCompletionRequest chatCompletionRequest = ChatCompletionRequest.builder()
                    .model(model)
                    .messages(messages)
                    .stream(true)
                    .n(1)
                    .maxTokens(maxTokens)
                    .logitBias(new HashMap<>())
                    .build();

            service.streamChatCompletion(chatCompletionRequest).doOnError(Throwable::printStackTrace).blockingForEach(chunk -> {
                chunk.getChoices().stream().map(choice -> choice.getMessage().getContent())
                        .filter(Objects::nonNull).findFirst().ifPresent(o -> {
                    try {
                        stringBuilder.append(new String(o.getBytes(Charset.defaultCharset())));
                    } catch (Exception e) {
                        throw new RuntimeException(e);
                    }
                });
            });
        } catch (Exception e) {
            StringWriter sw = new StringWriter();
            PrintWriter pw = new PrintWriter(sw);
            e.printStackTrace(pw);
            processLogService.log(meetingId,subMeetingId,"【大模型处理异常】:"+sw.toString());
            throw new RuntimeException(e);
        }
        service.shutdownExecutor();
        return stringBuilder.toString();
    }

    private String convertJSONToXml(String json) {
        JSONArray jsonArray = JSON.parseArray(json);;
        StringBuilder xmlBuilder = new StringBuilder();

        xmlBuilder.append("<root>");
        for (int i = 0; i < jsonArray.size(); i++) {
            JSONObject jsonObj = jsonArray.getJSONObject(i);

            String key = jsonObj.getString("key");
            String label = jsonObj.getString("keyName");
            String value = jsonObj.getString("value");

            xmlBuilder.append(String.format("<%s label=\"%s\">%s</%s>",
                    key, label, value, key));
        }
        xmlBuilder.append("</root>");

        return xmlBuilder.toString();
    }

    private static Map<String, Object> convertXmlToMap(String xml) throws Exception {

        XmlMapper xmlMapper = new XmlMapper();
        ObjectMapper objectMapper = new ObjectMapper();

        // 先将 XML 读取为 Map
        Map<?, ?> xmlMap = xmlMapper.readValue(xml, Map.class);

        // 转换为更标准的 Map<String, Object>
        Map<String,Object> map = objectMapper.convertValue(xmlMap, Map.class);
        //特殊处理map格式
        for (Map.Entry<String, Object> entry : map.entrySet()) {
            Map<String,Object> value = (Map<String, Object>) entry.getValue();
            //取出正确的value并设置
            String realValue = String.valueOf(value.get("")).replaceAll("^\\n+","");
            //内容段首移除换行，段末追加换行(会议名称结尾不换行
            entry.setValue(realValue.endsWith("\n") || "meeting_name".equals(entry.getKey()) || "meeting_purpose".equals(entry.getKey()) ? realValue : realValue + "\n");
        }
        return map;
    }

    /**
     * markdown转xml
     * @param markdown
     * @return
     */
    private String extractXmlFromMarkdown(String markdown,String meetingId, String subMeetingId) {
        StringBuffer sb;
        try {
            int start = markdown.indexOf("<");
            if(start == -1){
                processLogService.log(meetingId,subMeetingId,"markdown转xml失败，未输出正确的xml格式，markdown内容："+markdown);
            }
            int end = markdown.lastIndexOf(">") + 1;
            sb = new StringBuffer();
            sb.append("<root>");
            String xml = markdown.substring(start, end).trim().replaceAll("\n\n","\n");
            sb.append(xml);
            sb.append("</root>");
        } catch (Exception e) {
            log.info("markdown转xml，markdown->{}",markdown);
            throw new RuntimeException(e.getMessage());
        }
        return sb.toString();
    }

    private byte[] downloadFile(String url) {
        // 实现文件下载逻辑
        OkHttpClient client = new OkHttpClient();
        Request request = new Request.Builder().url(url).build();
        try {
            Response response = client.newCall(request).execute();
            if (!response.isSuccessful()) throw new IOException("Unexpected code " + response);
            return response.body().bytes();
        }catch (Exception e){
            throw new RuntimeException("下载文件失败", e);
        }
    }
}
